//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2013 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#include "inst/program_builder.hh"
#include "parse/asm-scan.hh"
#include "parse/asm-parse.hh"

// Export parsed file name.
char *filename = 0;

// Define exported parse functions.
namespace parse
{

  // Display the results of intermediate steps.
  bool trace_scan_p = 0;
  bool trace_parse_p = false;

  // Authorize unlimited registers.
  bool unlimited_regs_p = false;

  // Fill delay slot of native branch instructions with NOP.
  bool nop_after_branch_p = false;

  // Parse a MIPS asm file, return the corresponding abstract syntax.
  ::inst::Program*
  parse (const std::string &filename)
  {
    inst::ProgramBuilder program_builder;
    program_builder.reset();
    program_builder.set_nop_after_branch(nop_after_branch_p);

    std::string dummy(filename);
    yy::parser parser(dummy, program_builder);
    parser.set_debug_level(trace_parse_p);

    scan_open(filename, trace_scan_p);
    parser.parse();
    scan_close();

    return program_builder.get();
  }

} // namespace parse
