//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2004, 2005 Akim Demaille <akim@epita.fr> and
//                          Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TASK_LIBTASK_HH
# define TASK_LIBTASK_HH

# include "task/function_task.hh"
# include "task/boolean_task.hh"
# include "task/int_task.hh"
# include "task/string_task.hh"

# ifdef NOLIMIPS_CC_

#  define TASK_MODULE(Name)                     \
  const char module_name [] = Name;

#  define TASK_DECLARE(Option, Desc, Routine, Deps)             \
  extern void (Routine) (void);                                 \
  static task::FunctionTask task_##Routine(Routine,             \
                                           Option, module_name, \
                                           Desc, Deps)

#  define BOOLEAN_TASK_DECLARE(Option, Desc, Flag, Deps)        \
  bool Flag = false;                                            \
  static task::BooleanTask task_##Flag(Flag,                    \
                                       Option, module_name,     \
                                       Desc, Deps)

#  define INT_TASK_DECLARE(Option, Desc, Var, Default, Min, Max, Deps)  \
  int Var = Default;                                                    \
  static task::IntTask task_##Var(Var, Min, Max,                        \
                                  Option, module_name,                  \
                                  Desc, Deps)

#  define STRING_TASK_DECLARE(Option, Desc, Routine, Default, Deps)     \
  extern void (Routine) (const std::string &);                          \
  static task::StringTask task_##Routine(Routine,                       \
                                         Option, module_name,           \
                                         Desc, Deps)


# else // !NOLIMIPS_CC_

#  define TASK_MODULE(Name)
#  define TASK_DECLARE(Name, Help, Routine, Dependencies)       \
  extern void (Routine) (void)
#  define BOOLEAN_TASK_DECLARE(Name, Help, Flag, Dependencies)  \
  extern bool Flag
#  define INT_TASK_DECLARE(Option, Desc, Var, Default, Min, Max, Deps)  \
  extern int Var
#  define STRING_TASK_DECLARE(Option, Desc, Routine, Default, Deps)     \
  extern void (Routine) (const std::string &)

# endif // NOLIMIPS_CC_

#endif // !TASK_LIBTASK_HH
