//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef VM_MEMORY_HXX
# define VM_MEMORY_HXX

# include "vm/memory.hh"

# include "misc/contract.hh"

# include "inst/data_section.hh"

namespace vm
{

  //
  inline
  Memory::Memory(int stack_size):
    stack_top_(stack_bottom - stack_size + 4)
  {
    assertion(stack_size > 0);
    segments_[stack].resize(stack_size);
  }

  //
  inline void
  Memory::reset()
  {
    // DO NOT zero-ify the memory
  }

  inline int
  Memory::sbrk(int size)
  {
    precondition(size >= 0);
    int ptr = segments_[heap].size();
    if (size)
      // FIXME: check collision with stack
      segments_[heap].resize(size + segments_[heap].size());
    return ptr;
  }

  //
  inline int
  Memory::heap_size() const
  {
    return segments_[heap].size();
  }
  inline int
  Memory::stack_size() const
  {
    return segments_[stack].size();
  }

  inline int
  Memory::get_stack_top() const
  {
    return stack_top_;
  }

  //
  inline void
  Memory::store(const inst::DataSection& data_section)
  {
    segments_[heap].resize(data_section.size());
    for (int i = 0; i < data_section.size(); ++i)
      segments_[heap].store_byte(i, data_section.load_byte(i));
  }

  //
  inline void
  Memory::store_byte(segment_kind k, int offset, int b)
  {
    segments_[k].store_byte(offset, b);
  }
  inline void
  Memory::store_word(segment_kind k, int offset, int w)
  {
    segments_[k].store_word(offset, w);
  }

  //
  inline int
  Memory::load_byte(segment_kind k, int offset) const
  {
    return segments_[k].load_byte(offset);
  }
  inline int
  Memory::load_word(segment_kind k, int offset) const
  {
    return segments_[k].load_word(offset);
  }

} // namespace vm

#endif // !VM_MEMORY_HXX
